@@ -17,8 +17,14 @@ Classes and Functions:
 
 """
 import os
-from . import get_event_key_js,make_component,text_to_key
+from .FilesListManager import FilesListManager
+from .AbstractWindowManager import AbstractWindowManager
+from .abstract_gui import sg,get_event_key_js,make_component,text_to_key,ensure_nested_list,expandable
 from abstract_utilities import eatAll
+def capitalize(text):
+    if text:
+        text=text[0].upper()+text[1:]
+    return text
 class AbstractBrowser:
     """
     This class aims to provide a unified way of browsing through different sources.
@@ -37,11 +43,17 @@ class AbstractBrowser:
             self.window = make_component('Window','File/Folder Scanner', layout=self.get_scan_browser_layout())
         self.event = None
         self.values = None
-
+        self.files_list_mgr = FilesListManager()
+        self.mode_tracker={}
         self.scan_mode = "all"
         self.history = [os.getcwd()]
-        self.key_list = ['-BROWSER_LIST-','-CLEAR_LIST-','-ADD_TO_LIST-',"-FILES_BROWSER-","-DIR-","-DIRECTORY_BROWSER-","-FILES_LIST-","-SCAN_MODE_ALL-","-SELECT_HIGHLIGHTED-","-SCAN-", "-SELECT_HIGHLIGHTED-", "-MODE-", "-BROWSE_BACKWARDS-", "-BROWSE_FORWARDS-"]
-
+        self.modes = ['-SCAN_MODE_FILES-','-SCAN_MODE_FOLDERS-','-SCAN_MODE_ALL-']
+        self.key_list = ['-BROWSER_LIST-','-CLEAR_LIST-','-REMOVE_FROM_LIST-','-MOVE_UP_LIST-','-MOVE_DOWN_LIST-','-ADD_TO_LIST-',"-FILES_BROWSER-","-DIR-","-DIRECTORY_BROWSER-","-FILES_LIST-","-SCAN_MODE_ALL-","-SELECT_HIGHLIGHTED-","-SCAN-", "-SELECT_HIGHLIGHTED-", "-MODE-", "-BROWSE_BACKWARDS-", "-BROWSE_FORWARDS-"]+self.modes
+    def initiate_browser_window(self,window_name="Browser_Window",title="Abstract Browser"):
+        self.window_mgr = AbstractWindowManager()
+        self.window_name = self.window_mgr.add_window(window_name=window_name,title=title,layout=self.get_scan_browser_layout(),**expandable())
+        self.window = self.window_mgr.get_window_method(self.window_name)
+        self.window_mgr.while_window(window_name=self.window_name, event_handlers=self.handle_event)
     def handle_event(self,event,values,window):
           
         self.event,self.values,self.window=event,values,window
@@ -59,31 +71,40 @@ class AbstractBrowser:
             self.scan_results = self.scan_directory(directory, self.scan_mode)
             self.browse_update(key=self.event_key_js['-BROWSER_LIST-'],args={"values":self.scan_results})
     @staticmethod
-    def get_scan_browser_layout(section=None,extra_buttons=[]):
+    def get_scan_browser_layout(section=None, extra_buttons=[]):
         """
         Generate the layout for the file/folder scanning window.
 
         Returns:
-        --------
-        list:
-            A list of list of PySimpleGUI elements defining the window layout.
-        """
-        return [
-        [make_component("Text",'Directory to scan:'), make_component("InputText",os.getcwd(),key=text_to_key(text='dir',section=section)),
-         make_component("FolderBrowse","Folders", enable_events=True, key=text_to_key(text='directory browser',section=section)),
-         make_component("FilesBrowse","Files", enable_events=True, key=text_to_key(text='file browser',section=section)),
-         make_component("Checkbox",'all directory',default=True,key=text_to_key(text='scan mode all',section=section))], 
-        [make_component("Listbox",[],size=(50, 10),key=text_to_key(text='browser list',section=section),enable_events=True),
-         make_component("Listbox",[],size=(50, 10),key=text_to_key(text='files list',section=section),enable_events=True)],
-        [make_component("Button","Scan",key=text_to_key(text='scan',section=section),enable_events=True),
-        make_component("Button","<-",key=text_to_key(text='browse backwards',section=section),enable_events=True),
-        make_component("Button","All Scan Mode",key=text_to_key(text='mode',section=section),enable_events=True),
-        make_component("Button","->",key=text_to_key(text='browse forwards',section=section),enable_events=True),
-         make_component("Button","Add",key=text_to_key(text='add_to_list',section=section),enable_events=True),
-         make_component("Button","Clear",key=text_to_key(text='clear list',section=section),enable_events=True)]+extra_buttons
+            --------
+            list:
+                A list of list of PySimpleGUI elements defining the window layout.
+            """
+            
+        # More complex layout with additional elements
+        browser_buttons_layout = [
+            sg.Button('All Scan Mode', key=text_to_key(text='mode',section=section)),sg.Checkbox('All', default=True, key=text_to_key(text='scan mode all',section=section)),sg.Checkbox('Files', key=text_to_key(text='file browser',section=section)),sg.Checkbox('Folders', key=text_to_key(text='scan mode folders',section=section))
         ]
+        listboxes_layout=[]
+        listboxes_layout.append(sg.Frame('browser',layout=ensure_nested_list([[sg.Listbox(values=[], size=(50, 10), key=text_to_key(text='browser list',section=section), enable_events=True)],
+                                                                             [sg.Push(),sg.Button('<-', key=text_to_key(text='browse backwards',section=section)),
+                                                                              sg.Button('Scan', key=text_to_key(text='scan',section=section)),
+                                                                              sg.Button('->', key=text_to_key(text='browse forwards',section=section)),sg.Button('ADD', key=text_to_key(text='add_to_list',section=section)),sg.Push()]])))
+
+        listboxes_layout.append(sg.Frame('files',layout=ensure_nested_list([[sg.Listbox(values=[], size=(50, 10), key=text_to_key(text='files list',section=section), enable_events=True)],
+                                                                             [sg.Push(),sg.Button('UP', key=text_to_key(text='move up list',section=section)),sg.Button('DOWN', key=text_to_key(text='move down list',section=section)),
+                                                                              sg.Button('REMOVE', key=text_to_key(text='remove from list',section=section)),
+                                                                              sg.Button('Clear', key=text_to_key(text='clear list',section=section)),sg.Push()]])))
+
+
+        layout = [[sg.Text('Directory to scan:'), sg.InputText(os.getcwd(),key=text_to_key(text='dir',section=section)),sg.FolderBrowse('Folders', key=text_to_key(text='directory browser',section=section)),sg.FileBrowse('Files', key=text_to_key(text='file browser',section=section))],
+            [sg.Column([listboxes_layout,browser_buttons_layout])]]
+        if extra_buttons:
+            layout.append(extra_buttons)         
+        return layout
     def while_static(self,event_key_js,values,window):
         self.event_key_js,self.values,self.window=event_key_js,values,window
+        self.section_key = key = self.event_key_js['section']
         if self.event_key_js['found'] == "-FILES_BROWSER-":
             self.browse_update(key=self.event_key_js['-DIR-'],args={"value":self.values[self.event_key_js["-FILES_BROWSER-"]]})
         if self.event_key_js['found'] == "-DIRECTORY_BROWSER-":
@@ -93,23 +114,58 @@ class AbstractBrowser:
         if self.event_key_js['found'] == "-SELECT_HIGHLIGHTED-":
             if len(self.values[self.event_key_js['-BROWSER_LIST-']])>0:
                 self.browse_update(key=self.event_key_js['-DIR-'],args={"value":os.path.join(self.return_directory(), self.values[self.event_key_js['-BROWSER_LIST-']][0])})
+        if self.event_key_js['found'] in self.modes:
+            for mode in self.modes:
+                if self.event_key_js['found'] == mode:
+                    if self.values[self.event_key_js[mode]]:
+                        self.scan_mode = eatAll(mode[1:-1].split('_')[-1].lower(),['s'])
+                        self.window.Element(self.event_key_js['-MODE-']).update(text=f"{capitalize(self.scan_mode)} Scan Mode")
+                        for each_mode in self.modes:
+                            if each_mode != mode:
+                                self.window[self.event_key_js[each_mode]].update(False)
         if self.event_key_js['found'] == '-MODE-':
-            self.scan_mode = 'folder' if self.scan_mode == 'file' else 'file'
-            if self.values['-SCAN_MODE_ALL-']:
+            if self.event_key_js['-MODE-'] not in self.mode_tracker:
+                self.mode_tracker[self.event_key_js['-MODE-']]=[self.scan_mode,self.scan_mode]
+            if self.values[self.event_key_js['-SCAN_MODE_FILES-']]:
+                self.scan_mode = 'file'
+            elif self.values[self.event_key_js['-SCAN_MODE_FOLDERS-']]:
+                self.scan_mode = 'folder'
+            elif self.values[self.event_key_js['-SCAN_MODE_ALL-']]:
                 self.scan_mode='all'
-            self.window.Element(self.event_key_js['-MODE-']).update(text=f"{self.scan_mode[0].upper()}{self.scan_mode[1:]}  Scan Mode")
+            else:
+                modes = ['all','file','folder']
+                for mode in modes:
+                    if mode not in self.mode_tracker[self.event_key_js['-MODE-']]:
+                        self.scan_mode=mode
+                        break
+            if self.scan_mode != self.mode_tracker[self.event_key_js['-MODE-']][-1]:
+                self.mode_tracker[self.event_key_js['-MODE-']].append(self.scan_mode)
+                self.mode_tracker[self.event_key_js['-MODE-']]=self.mode_tracker[self.event_key_js['-MODE-']][1:]
+            self.window.Element(self.event_key_js['-MODE-']).update(text=f"{capitalize(self.scan_mode)} Scan Mode")
+            self.scan_it(self.return_directory())
+        if self.event_key_js['found'] in ['-MOVE_UP_LIST-','-MOVE_DOWN_LIST-','-REMOVE_FROM_LIST-']:
+            list_value = self.values[self.event_key_js['-FILES_LIST-']]
+            if list_value:
+                display_number=list_value[0]
+                if self.event_key_js['found'] == '-MOVE_UP_LIST-':
+                    display_values = self.files_list_mgr.move_up(self.section_key,display_number)
+                elif self.event_key_js['found'] == '-MOVE_DOWN_LIST-':
+                    display_values = self.files_list_mgr.move_down(self.section_key,display_number)
+                elif self.event_key_js['found'] == '-REMOVE_FROM_LIST-':
+                    display_values = self.files_list_mgr.remove_item(self.section_key,display_number)
+                self.window[self.event_key_js['-FILES_LIST-']].update(display_values)
         if self.event_key_js['found'] == '-ADD_TO_LIST-':
+            
+            self.files_list_mgr.check_files_list_key(self.section_key)
             file_list = self.values[self.event_key_js['-BROWSER_LIST-']]
-            files_list = self.values[self.event_key_js['-FILES_LIST-']] or []
+            path_dir = self.values[self.event_key_js['-DIR-']]
+            filename = file_list[0]
             if file_list:
-                files_list.append(file_list[0])
-            path_dir = self.values[self.event_key_js['-DIR-']].split(';')
-            for path in path_dir:
-                if path not in files_list:
-                    files_list.append(path)
-            self.window[self.event_key_js['-FILES_LIST-']].update(files_list)
+                file_path = os.path.join(path_dir,filename)
+                self.files_list_mgr.add_to_files_list_dict(self.section_key,file_path)
+                self.window[self.event_key_js['-FILES_LIST-']].update(self.files_list_mgr.return_display_names(self.section_key))
         if self.event_key_js['found'] == '-CLEAR_LIST-':
-            self.window[self.event_key_js['-FILES_LIST-']].update([])
+            self.window[self.event_key_js['-FILES_LIST-']].update(self.files_list_mgr.clear_list(key))
         if self.event_key_js['found'] == "-BROWSE_BACKWARDS-":
             # Navigate up to the parent directory
             if self.return_directory() not in self.history:
@@ -264,26 +320,7 @@ class AbstractBrowser:
                 self.window.Element("Cancel").update(text="Exit")
 
                 return "Cancel"
-    @staticmethod       
-    def save_entity(initial_folder:str=os.getcwd(), entity_type:str="Folder",entity_name:str="",default_prompt:str='Enter the new folder name:',save_data:any=None):
-        file_browser_visible = False
-        saved_prompt = "Folder Created!!"
-        if save_data:
-            js_browse_bridge["save_data"] = save_data
-            entity_type = "File"
-        if entity_type == "File":
-            saved_prompt = "File Saved!!"
-            if default_prompt == 'Enter the new folder name:':
-                default_prompt = 'Enter the new file name or choose from File Browser:'
-            file_browser_visible = True
-        layout = [
-            [get_gui_fun("Text", args={"text": default_prompt})],
-            [get_gui_fun("Input", args={"default_text":entity_name,"key": '-ENTITY_NAME-',"enable_events":True}),get_gui_fun("FileBrowse", args={"button_text":"File Browse","key":"-FOLDER_BROWSE-","initial_folder": initial_folder,"enable_events":True,"visible":file_browser_visible})],
-            [get_gui_fun("Text", args={"text": 'Select Parent Directory:'})],
-            [get_gui_fun("Input", args={"default_text": initial_folder, "key": "-PARENT_DIR-", "disabled": True,"enable_events":True}), 
-             get_gui_fun("FolderBrowse", args={"button_text":"Dir Browse","key":"-DIRECTORY_BROWSE-","initial_folder": initial_folder,"enable_events":True})],
-            [create_row_of_buttons('Create', {"button_text":'Cancel',"key":"Cancel","enable_events":True}),get_gui_fun("Push"),get_gui_fun("Input", args={"disabled":True,"default_text":entity_type,"key":"-ENTITY_TYPE-","size":(len(entity_type),1)}),get_gui_fun("Input", args={"default_text":"","key":"-FINAL_OUTPUT-","visible":False})],
-            [get_gui_fun("Text", args={"text": saved_prompt,"key":"-SAVE_PROMPT-","visible":False})]
-        ]
-        return self.values['-FINAL_OUTPUT-']
+
+
+
 
