@@ -1,4 +1,4 @@
-from abstract_gui import make_component,ensure_nested_list,get_screen_dimensions
+from .abstract_gui import make_component,ensure_nested_list,get_screen_dimensions
 from .RightClickManager import RightClickManager
 from abstract_utilities import create_new_name,make_list,ThreadManager
 class GUIManager:
